package com.dataart.viewer.product.service.impl;

import com.dataart.viewer.product.converter.FilterConverter;
import com.mysema.query.types.Predicate;
import com.mysema.query.types.expr.BooleanExpression;
import com.dataart.viewer.product.converter.SortConverter;
import com.dataart.viewer.product.model.impl.Product;
import com.dataart.viewer.product.model.impl.QProduct;
import com.dataart.viewer.product.util.PageUtil;
import com.dataart.viewer.product.mapper.ProductMapper;
import com.dataart.viewer.product.model.impl.ProductGroup;
import com.dataart.viewer.product.model.vo.FilterVO;
import com.dataart.viewer.product.model.vo.ProductGroupResultVO;
import com.dataart.viewer.product.model.vo.ProductVO;
import com.dataart.viewer.product.model.vo.SortVO;
import static com.dataart.viewer.product.model.vo.SortVO.Direction.ASC;
import static com.dataart.viewer.product.model.vo.SortVO.Direction.DESC;
import com.dataart.viewer.product.repository.ProductGroupRepository;
import com.dataart.viewer.product.repository.ProductRepository;
import com.dataart.viewer.product.service.ProductService;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.apache.commons.lang.Validate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @author volkodavav
 */
@Transactional(readOnly = true)
@Service
public class ProductServiceDefault implements ProductService, InitializingBean {

    protected Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ProductRepository productRepository;
    @Autowired
    private ProductGroupRepository productGroupRepository;
    @Autowired
    private ProductMapper productMapper;
    @Autowired
    private PageUtil pageUtil;
    @Autowired
    protected FilterConverter filterConverter;
    @Autowired
    protected SortConverter sortConverter;

    @Override
    public Page<ProductVO> getCurrentPage(int start, int limit, int total, String filterJson, String sortJson) {
        Validate.isTrue(start >= 0, "Start must be >= 0");
        Validate.isTrue(limit >= 0, "Limit must be >= 0");

        logger.info("Getting current page from {} with limit {}. Size: {}", start, limit, total);

        List<FilterVO> filter = filterConverter.fromJsonToList(filterJson);
        Sort sort = getSort(sortJson);

        Pageable pageable = pageUtil.createPageable(start, limit, total, sort);

        Page<ProductVO> result;
        if (filter != null && !filter.isEmpty()) {
            Predicate predicate = createPredicate(filter);
            result = productMapper.toVO(productRepository.findAll(predicate, pageable), pageable);
        } else {
            result = productMapper.toVO(productRepository.findAll(pageable), pageable);
        }

        return result;
    }

    @Override
    public List<ProductVO> findByGroupName(String groupName) {
        Validate.notEmpty(groupName, "Group name must not be null or empty");

        logger.info("Getting product by group name {}", groupName);
        List<ProductVO> result = productMapper.toVO(productRepository.findByGroup_Name(groupName));
        return result;
    }

    @Transactional
    @Override
    public void deleteAllProducts() {
        productRepository.deleteAll();
    }

    @Override
    public long count() {
        return productRepository.count();
    }

    @Transactional
    @Override
    public ProductVO save(ProductVO productVO) {
        Validate.notNull(productVO, "Input product must not be null");
        Validate.notEmpty(productVO.getGroup(), "Product group must not be null or empty");

        Product product = productMapper.toPersistence(productVO);
        String productGroup = productVO.getGroup();
        if (productGroup != null) {
            ProductGroup foundGroup = productGroupRepository.findByName(productGroup);
            if (foundGroup != null) {
                product.setGroup(foundGroup);
            }
        }

        Product savedProduct = productRepository.save(product);
        return productMapper.toVO(savedProduct);
    }

    @Override
    public List<ProductGroupResultVO> countProductsGroupedByGroupName() {
        Map<String, ProductGroupResultVO> resultMap = new HashMap<String, ProductGroupResultVO>();
        List<Object[]> aggregationResult = productRepository.countProductsGroupedByGroupName();

        for (Object[] objectResult : aggregationResult) {

            long number = (Long) objectResult[0];
            String groupName = (String) objectResult[1];
            addProductGroupResult(resultMap, groupName, number);
        }

        return new ArrayList<ProductGroupResultVO>(resultMap.values());
    }

    private void addProductGroupResult(Map<String, ProductGroupResultVO> resultMap, String groupName, long number) {
        ProductGroupResultVO value = resultMap.get(groupName);
        if (value == null) {
            resultMap.put(groupName, new ProductGroupResultVO(groupName, number));
        } else {
            value.addNumber(number);
        }
    }

    private Predicate createPredicate(final List<FilterVO> filterUI) {
        logger.info("Converting to predicates Filters {}", filterUI);

        if (filterUI == null) {
            logger.warn("No filters to convert");
            return null;
        }

        QProduct product = QProduct.product;
        BooleanExpression result = null;

        try {
            for (FilterVO filter : filterUI) {

                Product.FIELDS column = Product.FIELDS.valueOf(filter.getField().toUpperCase());
                BooleanExpression expression = null;

                switch (column) {
                    case NAME:
                        if (filter.getValue() != null && !"".equals(filter.getValue())) {
                            expression = product.name.like("%" + filter.getValue() + "%");
                        }
                        break;
                    case PRICE:
                        if (filter.getValue() != null && !"".equals(filter.getValue())) {
                            BigDecimal value = BigDecimal.valueOf(Double.valueOf(filter.getValue().toString()));
                            FilterVO.Comparison comparison = filter.getComparison();
                            switch (comparison) {
                                case EQ:
                                    expression = product.price.eq(value);
                                    break;
                                case GT:
                                    expression = product.price.gt(value);
                                    break;
                                case LT:
                                    expression = product.price.lt(value);
                                    break;
                            }
                        }
                        break;
                    case GROUP:
                        if (filter.getValue() != null && !"".equals(filter.getValue())) {
                            expression = product.group.name.like("%" + filter.getValue() + "%");
                        }
                        break;
                }
                if (expression != null) {
                    if (result != null) {
                        result = result.and(expression);
                    } else {
                        result = expression;
                    }
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
        }
        logger.info("Result Predicate {}", result);

        return result;
    }

    private Sort getSort(String sortJson) {
        List<SortVO> sortUI = sortConverter.fromJsonToList(sortJson);
        if (sortUI == null || sortUI.isEmpty()) {
            return null;
        }

        List<Sort.Order> orders = new ArrayList<Sort.Order>();

        for (SortVO sortUIElement : sortUI) {
            String propertyPath = sortUIElement.getProperty();
            Product.FIELDS column = Product.FIELDS.valueOf(sortUIElement.getProperty().toUpperCase());
            switch (column) {
                case GROUP:
                    propertyPath = "group.name";
            }

            switch (sortUIElement.getDirection()) {
                case ASC:
                    orders.add(new Sort.Order(Sort.Direction.ASC, propertyPath));
                    break;
                case DESC:
                    orders.add(new Sort.Order(Sort.Direction.DESC, propertyPath));
                    break;
            }
        }

        return new Sort(orders);
    }

    public void setProductRepository(ProductRepository productRepository) {
        this.productRepository = productRepository;
    }

    public void setProductGroupRepository(ProductGroupRepository productGroupRepository) {
        this.productGroupRepository = productGroupRepository;
    }

    public void setProductMapper(ProductMapper productMapper) {
        this.productMapper = productMapper;
    }

    public void setPageUtil(PageUtil pageUtil) {
        this.pageUtil = pageUtil;
    }

    public void setFilterConverter(FilterConverter filterConverter) {
        this.filterConverter = filterConverter;
    }

    public void setSortConverter(SortConverter sortConverter) {
        this.sortConverter = sortConverter;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        Validate.notNull(pageUtil, "pageUtil expected");
        Validate.notNull(productRepository, "productRepository expected");
        Validate.notNull(productGroupRepository, "productGroupRepository expected");
        Validate.notNull(productMapper, "productMapper expected");
        Validate.notNull(filterConverter, "filterConverter expected");
        Validate.notNull(sortConverter, "sortConverter expected");
    }
}
