package com.dataart.viewer.product.service.impl;

import com.dataart.viewer.product.converter.FilterConverter;
import com.dataart.viewer.product.converter.JsonConverter;
import com.dataart.viewer.product.converter.SortConverter;
import com.dataart.viewer.product.mapper.ProductMapper;
import com.dataart.viewer.product.model.impl.Product;
import com.dataart.viewer.product.model.impl.ProductGroup;
import com.dataart.viewer.product.model.vo.ProductGroupResultVO;
import com.dataart.viewer.product.model.vo.ProductVO;
import com.dataart.viewer.product.repository.ProductGroupRepository;
import com.dataart.viewer.product.repository.ProductRepository;
import com.dataart.viewer.product.util.PageUtil;
import static org.mockito.Mockito.*;
import static org.junit.Assert.*;
import com.mysema.query.types.Predicate;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import org.junit.Test;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

/**
 *
 * @author volkodavav
 */
public class ProductServiceDefaultTest {

    @Test
    public void testGetCurrentPageWithFilterAndSort() throws Exception {
        String FAKE_PRODUCT_NAME = "fakeProduct";
        String FAKE_GROUP_NAME = "fakeGroup";
        ProductGroup FAKE_PRODUCT_GROUP = new ProductGroup(FAKE_GROUP_NAME);
        BigDecimal productPrice = BigDecimal.valueOf(10);
        Product FAKE_PRODUCT = new Product(FAKE_PRODUCT_NAME, productPrice, FAKE_PRODUCT_GROUP);
        int start = 0;
        int limit = 10;
        int total = -1;
        String filterJson = "[{\"type\":\"string\",\"value\":\"Desktops\",\"field\":\"group\"}]";
        String sortJson = "[{\"property\":\"name\",\"direction\":\"ASC\"}]";

        Pageable FAKE_PAGEABLE = new PageRequest(1, 1);

        PageUtil FAKE_PAGE_UTIL = mock(PageUtil.class);
        when(FAKE_PAGE_UTIL.createPageable(eq(start), eq(limit), eq(total), any(Sort.class))).thenReturn(FAKE_PAGEABLE);

        ProductRepository productRepository = mock(ProductRepository.class);
        when(productRepository.findAll(any(Predicate.class), eq(FAKE_PAGEABLE))).thenReturn(new PageImpl<Product>(Arrays.asList(FAKE_PRODUCT)));

        JsonConverter jsonConverter = new JsonConverter();
        FilterConverter filterConverter = new FilterConverter();
        filterConverter.setJsonConverter(jsonConverter);
        filterConverter.afterPropertiesSet();

        SortConverter sortConverter = new SortConverter();
        sortConverter.setJsonConverter(jsonConverter);
        sortConverter.afterPropertiesSet();

        ProductMapper productMapper = new ProductMapper();

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(productRepository);
        productService.setPageUtil(FAKE_PAGE_UTIL);
        productService.setFilterConverter(filterConverter);
        productService.setSortConverter(sortConverter);
        productService.setProductMapper(productMapper);
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        Page<ProductVO> currentPage = productService.getCurrentPage(start, limit, total, filterJson, sortJson);
        ProductVO productVO = currentPage.getContent().get(0);

        verify(productRepository, times(1)).findAll(any(Predicate.class), eq(FAKE_PAGEABLE));

        assertNotNull(productVO);
        assertEquals(FAKE_PRODUCT_NAME, productVO.getName());
        assertEquals(FAKE_GROUP_NAME, productVO.getGroup());
        assertEquals(productPrice, productVO.getPrice());
    }

    @Test(expected = IllegalArgumentException.class)
    public void testGetCurrentPageWithNotValidStart() throws Exception {
        int start = -1;

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(mock(ProductRepository.class));
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(mock(ProductMapper.class));
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        Page<ProductVO> currentPage = productService.getCurrentPage(start, 1, 1, null, null);
        assertTrue("Assert shouldn't be called", false);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testGetCurrentPageWithNotValidLimit() throws Exception {
        int limit = -1;

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(mock(ProductRepository.class));
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(mock(ProductMapper.class));
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        Page<ProductVO> currentPage = productService.getCurrentPage(1, limit, 1, null, null);
        assertTrue("Assert shouldn't be called", false);
    }

    @Test
    public void testGetCurrentPageWithoutFilterAndSort() throws Exception {
        String FAKE_PRODUCT_NAME = "fakeProduct";
        String FAKE_GROUP_NAME = "fakeGroup";
        ProductGroup FAKE_PRODUCT_GROUP = new ProductGroup(FAKE_GROUP_NAME);
        BigDecimal productPrice = BigDecimal.valueOf(10);
        Product FAKE_PRODUCT = new Product(FAKE_PRODUCT_NAME, productPrice, FAKE_PRODUCT_GROUP);
        int start = 0;
        int limit = 10;
        int total = -1;
        String filterJson = null;
        String sortJson = null;

        Pageable FAKE_PAGEABLE = new PageRequest(1, 1);

        PageUtil FAKE_PAGE_UTIL = mock(PageUtil.class);
        when(FAKE_PAGE_UTIL.createPageable(eq(start), eq(limit), eq(total), any(Sort.class))).thenReturn(FAKE_PAGEABLE);

        ProductRepository productRepository = mock(ProductRepository.class);
        when(productRepository.findAll(eq(FAKE_PAGEABLE))).thenReturn(new PageImpl<Product>(Arrays.asList(FAKE_PRODUCT)));

        ProductMapper productMapper = new ProductMapper();

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(productRepository);
        productService.setPageUtil(FAKE_PAGE_UTIL);
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(productMapper);
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        Page<ProductVO> currentPage = productService.getCurrentPage(start, limit, total, filterJson, sortJson);
        ProductVO productVO = currentPage.getContent().get(0);

        verify(productRepository, times(1)).findAll(eq(FAKE_PAGEABLE));

        assertNotNull(productVO);
        assertEquals(FAKE_PRODUCT_NAME, productVO.getName());
        assertEquals(FAKE_GROUP_NAME, productVO.getGroup());
        assertEquals(productPrice, productVO.getPrice());
    }

    @Test
    public void testFindByGroupName() throws Exception {
        String FAKE_PRODUCT_NAME = "fakeProduct";
        String FAKE_GROUP_NAME = "fakeGroup";
        ProductGroup FAKE_PRODUCT_GROUP = new ProductGroup(FAKE_GROUP_NAME);
        BigDecimal productPrice = BigDecimal.valueOf(10);
        Product FAKE_PRODUCT = new Product(FAKE_PRODUCT_NAME, productPrice, FAKE_PRODUCT_GROUP);

        ProductMapper productMapper = new ProductMapper();

        ProductRepository productRepository = mock(ProductRepository.class);
        when(productRepository.findByGroup_Name(eq(FAKE_GROUP_NAME))).thenReturn(Arrays.asList(FAKE_PRODUCT));

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(productRepository);
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(productMapper);
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        List<ProductVO> products = productService.findByGroupName(FAKE_GROUP_NAME);
        ProductVO productVO = products.get(0);

        verify(productRepository, times(1)).findByGroup_Name(eq(FAKE_GROUP_NAME));

        assertNotNull(productVO);
        assertEquals(FAKE_PRODUCT_NAME, productVO.getName());
        assertEquals(FAKE_GROUP_NAME, productVO.getGroup());
        assertEquals(productPrice, productVO.getPrice());
    }

    @Test(expected = IllegalArgumentException.class)
    public void testFindByGroupNameWithNullAsGroupName() throws Exception {
        String groupName = null;

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(mock(ProductRepository.class));
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(mock(ProductMapper.class));
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        List<ProductVO> products = productService.findByGroupName(groupName);
        assertTrue("Assert shouldn't be called", false);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testFindByGroupNameWithEmptyStringAsGroupName() throws Exception {
        String groupName = "";

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(mock(ProductRepository.class));
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(mock(ProductMapper.class));
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        List<ProductVO> products = productService.findByGroupName(groupName);
        assertTrue("Assert shouldn't be called", false);
    }

    @Test
    public void testDeleteAllProducts() throws Exception {
        ProductRepository productRepository = mock(ProductRepository.class);

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(productRepository);
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(mock(ProductMapper.class));
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();
        productService.deleteAllProducts();

        verify(productRepository, times(1)).deleteAll();
    }

    @Test
    public void testCount() throws Exception {
        long EXPECTED_COUNT = 10L;

        ProductRepository productRepository = mock(ProductRepository.class);
        when(productRepository.count()).thenReturn(EXPECTED_COUNT);

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(productRepository);
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(mock(ProductMapper.class));
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();
        long resultCount = productService.count();

        assertEquals(EXPECTED_COUNT, resultCount);

        verify(productRepository, times(1)).count();
    }

    @Test
    public void testSave() throws Exception {
        String FAKE_PRODUCT_NAME = "fakeProduct";
        String FAKE_GROUP_NAME = "fakeGroup";
        ProductGroup FAKE_PRODUCT_GROUP = new ProductGroup(FAKE_GROUP_NAME);
        BigDecimal productPrice = BigDecimal.valueOf(10);
        Product FAKE_PRODUCT = new Product(FAKE_PRODUCT_NAME, productPrice, FAKE_PRODUCT_GROUP);
        ProductVO FAKE_PRODUCT_VO = new ProductVO(FAKE_PRODUCT_NAME, productPrice, FAKE_GROUP_NAME);

        ProductMapper productMapper = mock(ProductMapper.class);
        when(productMapper.toPersistence(FAKE_PRODUCT_VO)).thenReturn(FAKE_PRODUCT);
        when(productMapper.toVO(FAKE_PRODUCT)).thenReturn(FAKE_PRODUCT_VO);

        ProductRepository productRepository = mock(ProductRepository.class);
        when(productRepository.save(eq(FAKE_PRODUCT))).thenReturn(FAKE_PRODUCT);

        ProductGroupRepository productGroupRepository = mock(ProductGroupRepository.class);
        when(productGroupRepository.findByName(eq(FAKE_GROUP_NAME))).thenReturn(FAKE_PRODUCT_GROUP);

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(productRepository);
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(productMapper);
        productService.setProductGroupRepository(productGroupRepository);

        productService.afterPropertiesSet();

        ProductVO savedProductVO = productService.save(FAKE_PRODUCT_VO);

        verify(productGroupRepository, times(1)).findByName(FAKE_GROUP_NAME);

        assertNotNull(savedProductVO);
        assertEquals(FAKE_PRODUCT_NAME, savedProductVO.getName());
        assertEquals(FAKE_GROUP_NAME, savedProductVO.getGroup());
        assertEquals(productPrice, savedProductVO.getPrice());
    }

    @Test(expected = IllegalArgumentException.class)
    public void testSaveWithNullProduct() throws Exception {
        ProductVO productVO = null;

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(mock(ProductRepository.class));
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(mock(ProductMapper.class));
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        ProductVO savedProductVO = productService.save(productVO);
        assertTrue("Assert shouldn't be called", false);
    }

    @Test
    public void testCountProductsGroupedByGroupName() throws Exception {
        String FAKE_GROUP_NAME = "fakeGroup";
        long FAKE_GROUP_NUMBER = 10L;
        List<Object[]> FAKE_RESULTS = new ArrayList<Object[]>();
        FAKE_RESULTS.add(new Object[]{FAKE_GROUP_NUMBER, FAKE_GROUP_NAME});

        ProductRepository productRepository = mock(ProductRepository.class);
        when(productRepository.countProductsGroupedByGroupName()).thenReturn(FAKE_RESULTS);

        ProductServiceDefault productService = new ProductServiceDefault();
        productService.setProductRepository(productRepository);
        productService.setPageUtil(mock(PageUtil.class));
        productService.setFilterConverter(mock(FilterConverter.class));
        productService.setSortConverter(mock(SortConverter.class));
        productService.setProductMapper(mock(ProductMapper.class));
        productService.setProductGroupRepository(mock(ProductGroupRepository.class));

        productService.afterPropertiesSet();

        List<ProductGroupResultVO> results = productService.countProductsGroupedByGroupName();
        ProductGroupResultVO resultVO = results.get(0);

        verify(productRepository, times(1)).countProductsGroupedByGroupName();

        assertNotNull(resultVO);
        assertEquals(FAKE_GROUP_NAME, resultVO.getGroup());
        assertEquals(FAKE_GROUP_NUMBER, resultVO.getNumber());
    }
}
